@@ -65,12 +65,6 @@ enum
   LAST_SIGNAL
 };
 
-static const GstElementDetails gst_cmml_enc_details =
-GST_ELEMENT_DETAILS ("CMML streams encoder",
-    "Codec/Encoder",
-    "Encodes CMML streams",
-    "Alessandro Decina <alessandro@nnva.org>");
-
 static GstStaticPadTemplate gst_cmml_enc_src_factory =
 GST_STATIC_PAD_TEMPLATE ("src",
     GST_PAD_SRC,
@@ -118,7 +112,9 @@ gst_cmml_enc_base_init (gpointer g_class)
       gst_static_pad_template_get (&gst_cmml_enc_sink_factory));
   gst_element_class_add_pad_template (element_class,
       gst_static_pad_template_get (&gst_cmml_enc_src_factory));
-  gst_element_class_set_details (element_class, &gst_cmml_enc_details);
+  gst_element_class_set_details_simple (element_class, "CMML streams encoder",
+      "Codec/Encoder",
+      "Encodes CMML streams", "Alessandro Decina <alessandro@nnva.org>");
 }
 
 static void
@@ -134,19 +130,22 @@ gst_cmml_enc_class_init (GstCmmlEncClass * enc_class)
       g_param_spec_int64 ("granule-rate-numerator",
           "Granulerate numerator",
           "Granulerate numerator",
-          0, G_MAXINT64, 1000, G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
+          0, G_MAXINT64, 1000,
+          G_PARAM_READWRITE | G_PARAM_CONSTRUCT | G_PARAM_STATIC_STRINGS));
 
   g_object_class_install_property (klass, GST_CMML_ENC_GRANULERATE_D,
       g_param_spec_int64 ("granule-rate-denominator",
           "Granulerate denominator",
           "Granulerate denominator",
-          0, G_MAXINT64, 1, G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
+          0, G_MAXINT64, 1,
+          G_PARAM_READWRITE | G_PARAM_CONSTRUCT | G_PARAM_STATIC_STRINGS));
 
   g_object_class_install_property (klass, GST_CMML_ENC_GRANULESHIFT,
       g_param_spec_uchar ("granule-shift",
           "Granuleshift",
           "The number of lower bits to use for partitioning a granule position",
-          0, 64, 32, G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
+          0, 64, 32,
+          G_PARAM_READWRITE | G_PARAM_CONSTRUCT | G_PARAM_STATIC_STRINGS));
 
   GST_ELEMENT_CLASS (klass)->change_state = gst_cmml_enc_change_state;
 }
@@ -457,7 +456,7 @@ gst_cmml_enc_parse_tag_head (GstCmmlEnc * enc, GstCmmlTagHead * head)
     enc->flow_return = gst_cmml_enc_push (enc, buffer);
     headers = g_list_delete_link (headers, headers);
 
-    if (GST_FLOW_IS_FATAL (enc->flow_return))
+    if (enc->flow_return != GST_FLOW_OK)
       goto push_error;
   }
 
@@ -562,7 +561,7 @@ gst_cmml_enc_push_clip (GstCmmlEnc * enc, GstCmmlTagClip * clip,
   GST_BUFFER_TIMESTAMP (buffer) = clip->start_time;
 
   res = gst_cmml_enc_push (enc, buffer);
-  if (GST_FLOW_IS_FATAL (res))
+  if (res != GST_FLOW_OK)
     goto done;
 
   if (clip->end_time != GST_CLOCK_TIME_NONE) {
@@ -589,11 +588,8 @@ gst_cmml_enc_push (GstCmmlEnc * enc, GstBuffer * buffer)
 {
   GstFlowReturn res;
 
-  /* FIXME: hack to make oggmux flush every cmml tag in its own page */
-  GST_BUFFER_DURATION (buffer) = G_MAXINT64;
-
   res = gst_pad_push (enc->srcpad, buffer);
-  if (GST_FLOW_IS_FATAL (res))
+  if (res != GST_FLOW_OK)
     GST_WARNING_OBJECT (enc, "push returned: %s", gst_flow_get_name (res));
 
   return res;
