@@ -26,24 +26,20 @@
  * SECTION:element-esdsink
  * @see_also: #GstAlsaSink, #GstAutoAudioSink
  *
- * <refsect2>
- * <para>
  * This element outputs sound to an already-running Enlightened Sound Daemon
  * (ESound Daemon, esd). Note that a sound daemon will never be auto-spawned
  * through this element (regardless of the system configuration), since this
  * is actively prevented by the element. If you must use esd, you need to
  * make sure it is started automatically with your session or otherwise.
- * </para>
- * <para>
+ *
  * TODO: insert some comments about how sucky esd is and that all the cool
  * kids use pulseaudio or whatever these days.
- * </para>
- * <para>
- * Simple example pipeline that plays an Ogg/Vorbis file via esd:
- * <programlisting>
+ *
+ * <refsect2>
+ * <title>Example launch line</title>
+ * |[
  * gst-launch -v filesrc location=foo.ogg ! decodebin ! audioconvert ! audioresample ! esdsink
- * </programlisting>
- * </para>
+ * ]| play an Ogg/Vorbis audio file via esd
  * </refsect2>
  */
 
@@ -66,13 +62,6 @@
 GST_DEBUG_CATEGORY_EXTERN (esd_debug);
 #define GST_CAT_DEFAULT esd_debug
 
-/* elementfactory information */
-static const GstElementDetails esdsink_details =
-GST_ELEMENT_DETAILS ("Esound audio sink",
-    "Sink/Audio",
-    "Plays audio to an esound server",
-    "Arwed von Merkatz <v.merkatz@gmx.net>");
-
 enum
 {
   PROP_0,
@@ -124,7 +113,10 @@ gst_esdsink_base_init (gpointer g_class)
 
   gst_element_class_add_pad_template (element_class,
       gst_static_pad_template_get (&sink_factory));
-  gst_element_class_set_details (element_class, &esdsink_details);
+  gst_element_class_set_details_simple (element_class, "Esound audio sink",
+      "Sink/Audio",
+      "Plays audio to an esound server",
+      "Arwed von Merkatz <v.merkatz@gmx.net>");
 }
 
 static void
@@ -132,12 +124,10 @@ gst_esdsink_class_init (GstEsdSinkClass * klass)
 {
   GObjectClass *gobject_class;
   GstBaseSinkClass *gstbasesink_class;
-  GstBaseAudioSinkClass *gstbaseaudiosink_class;
   GstAudioSinkClass *gstaudiosink_class;
 
   gobject_class = (GObjectClass *) klass;
   gstbasesink_class = (GstBaseSinkClass *) klass;
-  gstbaseaudiosink_class = (GstBaseAudioSinkClass *) klass;
   gstaudiosink_class = (GstAudioSinkClass *) klass;
 
   parent_class = g_type_class_peek_parent (klass);
@@ -160,7 +150,8 @@ gst_esdsink_class_init (GstEsdSinkClass * klass)
   /* default value is filled in the _init method */
   g_object_class_install_property (gobject_class, PROP_HOST,
       g_param_spec_string ("host", "Host",
-          "The host running the esound daemon", NULL, G_PARAM_READWRITE));
+          "The host running the esound daemon", NULL,
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 }
 
 static void
@@ -413,7 +404,7 @@ write_error:
   {
     GST_ELEMENT_ERROR (esdsink, RESOURCE, WRITE,
         ("Failed to write data to the esound daemon"), GST_ERROR_SYSTEM);
-    return 0;
+    return -1;
   }
 }
 
