@@ -30,12 +30,6 @@
 #include <gst/video/video.h>
 #include <gst/gst-i18n-plugin.h>
 
-static const GstElementDetails gst_pngdec_details =
-GST_ELEMENT_DETAILS ("PNG image decoder",
-    "Codec/Decoder/Image",
-    "Decode a png video frame to a raw image",
-    "Wim Taymans <wim@fluendo.com>");
-
 GST_DEBUG_CATEGORY_STATIC (pngdec_debug);
 #define GST_CAT_DEFAULT pngdec_debug
 
@@ -111,7 +105,10 @@ gst_pngdec_base_init (gpointer g_class)
       gst_static_pad_template_get (&gst_pngdec_src_pad_template));
   gst_element_class_add_pad_template (element_class,
       gst_static_pad_template_get (&gst_pngdec_sink_pad_template));
-  gst_element_class_set_details (element_class, &gst_pngdec_details);
+  gst_element_class_set_details_simple (element_class, "PNG image decoder",
+      "Codec/Decoder/Image",
+      "Decode a png video frame to a raw image",
+      "Wim Taymans <wim@fluendo.com>");
 }
 
 static void
@@ -189,7 +186,7 @@ user_info_callback (png_structp png_ptr, png_infop info)
   size_t buffer_size;
   GstBuffer *buffer = NULL;
 
-  pngdec = GST_PNGDEC (png_ptr->io_ptr);
+  pngdec = GST_PNGDEC (png_get_io_ptr (png_ptr));
 
   GST_LOG ("info ready");
 
@@ -228,7 +225,7 @@ user_endrow_callback (png_structp png_ptr, png_bytep new_row,
 {
   GstPngDec *pngdec = NULL;
 
-  pngdec = GST_PNGDEC (png_ptr->io_ptr);
+  pngdec = GST_PNGDEC (png_get_io_ptr (png_ptr));
 
   /* FIXME: implement interlaced pictures */
 
@@ -275,7 +272,7 @@ user_end_callback (png_structp png_ptr, png_infop info)
 {
   GstPngDec *pngdec = NULL;
 
-  pngdec = GST_PNGDEC (png_ptr->io_ptr);
+  pngdec = GST_PNGDEC (png_get_io_ptr (png_ptr));
 
   GST_LOG_OBJECT (pngdec, "and we are done reading this image");
 
@@ -310,7 +307,7 @@ user_read_data (png_structp png_ptr, png_bytep data, png_size_t length)
   GstFlowReturn ret = GST_FLOW_OK;
   guint size;
 
-  pngdec = GST_PNGDEC (png_ptr->io_ptr);
+  pngdec = GST_PNGDEC (png_get_io_ptr (png_ptr));
 
   GST_LOG ("reading %" G_GSIZE_FORMAT " bytes of data at offset %d", length,
       pngdec->offset);
@@ -338,12 +335,15 @@ pause:
     GST_INFO_OBJECT (pngdec, "pausing task, reason %s",
         gst_flow_get_name (ret));
     gst_pad_pause_task (pngdec->sinkpad);
-    if (GST_FLOW_IS_FATAL (ret) || ret == GST_FLOW_NOT_LINKED) {
+    if (ret == GST_FLOW_UNEXPECTED) {
+      gst_pad_push_event (pngdec->srcpad, gst_event_new_eos ());
+    } else if (ret < GST_FLOW_UNEXPECTED || ret == GST_FLOW_NOT_LINKED) {
       GST_ELEMENT_ERROR (pngdec, STREAM, FAILED,
           (_("Internal data stream error.")),
           ("stream stopped, reason %s", gst_flow_get_name (ret)));
       gst_pad_push_event (pngdec->srcpad, gst_event_new_eos ());
     }
+    png_error (png_ptr, "Internal data stream error.");
     return;
   }
 short_buffer:
@@ -397,7 +397,11 @@ gst_pngdec_caps_create_and_set (GstPngDec * pngdec)
     png_set_gray_to_rgb (pngdec->png);
     if (bpc < 8) {              /* Convert to 8 bits */
       GST_LOG_OBJECT (pngdec, "converting grayscale image to 8 bits");
+#if PNG_LIBPNG_VER < 10400
       png_set_gray_1_2_4_to_8 (pngdec->png);
+#else
+      png_set_expand_gray_1_2_4_to_8 (pngdec->png);
+#endif
     }
   }
 
@@ -543,7 +547,9 @@ pause:
     GST_INFO_OBJECT (pngdec, "pausing task, reason %s",
         gst_flow_get_name (ret));
     gst_pad_pause_task (pngdec->sinkpad);
-    if (GST_FLOW_IS_FATAL (ret) || ret == GST_FLOW_NOT_LINKED) {
+    if (ret == GST_FLOW_UNEXPECTED) {
+      gst_pad_push_event (pngdec->srcpad, gst_event_new_eos ());
+    } else if (ret == GST_FLOW_NOT_LINKED || ret < GST_FLOW_UNEXPECTED) {
       GST_ELEMENT_ERROR (pngdec, STREAM, FAILED,
           (_("Internal data stream error.")),
           ("stream stopped, reason %s", gst_flow_get_name (ret)));
