@@ -39,6 +39,8 @@
 
 #include <string.h>
 #include <gst/base/gstbytereader.h>
+#include <gst/pbutils/descriptions.h>
+#include <gst/pbutils/pbutils.h>
 
 static GstStaticPadTemplate flv_sink_template = GST_STATIC_PAD_TEMPLATE ("sink",
     GST_PAD_SINK,
@@ -572,21 +574,20 @@ gst_flv_demux_audio_negotiate (GstFlvDemux * demux, guint32 codec_tag,
     guint32 rate, guint32 channels, guint32 width)
 {
   GstCaps *caps = NULL;
-  const gchar *codec_name = NULL;
+  gchar *codec_name = NULL;
   gboolean ret = FALSE;
+  guint adjusted_rate = rate;
 
   switch (codec_tag) {
     case 1:
       caps = gst_caps_new_simple ("audio/x-adpcm", "layout", G_TYPE_STRING,
           "swf", NULL);
-      codec_name = "Shockwave ADPCM";
       break;
     case 2:
     case 14:
       caps = gst_caps_new_simple ("audio/mpeg",
           "mpegversion", G_TYPE_INT, 1, "layer", G_TYPE_INT, 3,
           "parsed", G_TYPE_BOOLEAN, TRUE, NULL);
-      codec_name = "MPEG 1 Audio, Layer 3 (MP3)";
       break;
     case 0:
     case 3:
@@ -597,30 +598,44 @@ gst_flv_demux_audio_negotiate (GstFlvDemux * demux, guint32 codec_tag,
           "endianness", G_TYPE_INT, G_LITTLE_ENDIAN,
           "signed", G_TYPE_BOOLEAN, (width == 8) ? FALSE : TRUE,
           "width", G_TYPE_INT, width, "depth", G_TYPE_INT, width, NULL);
-      codec_name = "Raw Audio";
       break;
     case 4:
     case 5:
     case 6:
       caps = gst_caps_new_simple ("audio/x-nellymoser", NULL);
-      codec_name = "Nellymoser ASAO";
       break;
     case 10:
+    {
+      /* use codec-data to extract and verify samplerate */
+      if (demux->audio_codec_data &&
+          GST_BUFFER_SIZE (demux->audio_codec_data) >= 2) {
+        gint freq_index;
+
+        freq_index =
+            ((GST_READ_UINT16_BE (GST_BUFFER_DATA (demux->audio_codec_data))));
+        freq_index = (freq_index & 0x0780) >> 7;
+        adjusted_rate =
+            gst_codec_utils_aac_get_sample_rate_from_index (freq_index);
+
+        if (adjusted_rate && (rate != adjusted_rate)) {
+          GST_LOG_OBJECT (demux, "Ajusting AAC sample rate %d -> %d", rate,
+              adjusted_rate);
+        } else {
+          adjusted_rate = rate;
+        }
+      }
       caps = gst_caps_new_simple ("audio/mpeg",
           "mpegversion", G_TYPE_INT, 4, "framed", G_TYPE_BOOLEAN, TRUE, NULL);
-      codec_name = "AAC";
       break;
+    }
     case 7:
       caps = gst_caps_new_simple ("audio/x-alaw", NULL);
-      codec_name = "A-Law";
       break;
     case 8:
       caps = gst_caps_new_simple ("audio/x-mulaw", NULL);
-      codec_name = "Mu-Law";
       break;
     case 11:
       caps = gst_caps_new_simple ("audio/x-speex", NULL);
-      codec_name = "Speex";
       break;
     default:
       GST_WARNING_OBJECT (demux, "unsupported audio codec tag %u", codec_tag);
@@ -631,8 +646,8 @@ gst_flv_demux_audio_negotiate (GstFlvDemux * demux, guint32 codec_tag,
     goto beach;
   }
 
-  gst_caps_set_simple (caps,
-      "rate", G_TYPE_INT, rate, "channels", G_TYPE_INT, channels, NULL);
+  gst_caps_set_simple (caps, "rate", G_TYPE_INT, adjusted_rate,
+      "channels", G_TYPE_INT, channels, NULL);
 
   if (demux->audio_codec_data) {
     gst_caps_set_simple (caps, "codec_data", GST_TYPE_BUFFER,
@@ -642,17 +657,20 @@ gst_flv_demux_audio_negotiate (GstFlvDemux * demux, guint32 codec_tag,
   ret = gst_pad_set_caps (demux->audio_pad, caps);
 
   if (G_LIKELY (ret)) {
-    /* Store the caps we have set */
+    /* Store the caps we got from tags */
     demux->audio_codec_tag = codec_tag;
     demux->rate = rate;
     demux->channels = channels;
     demux->width = width;
 
+    codec_name = gst_pb_utils_get_codec_description (caps);
+
     if (codec_name) {
       if (demux->taglist == NULL)
         demux->taglist = gst_tag_list_new ();
       gst_tag_list_add (demux->taglist, GST_TAG_MERGE_REPLACE,
           GST_TAG_AUDIO_CODEC, codec_name, NULL);
+      g_free (codec_name);
     }
 
     GST_DEBUG_OBJECT (demux->audio_pad, "successfully negotiated caps %"
@@ -668,6 +686,28 @@ beach:
   return ret;
 }
 
+static void
+gst_flv_demux_push_tags (GstFlvDemux * demux)
+{
+  if (demux->has_audio && !demux->audio_pad) {
+    GST_DEBUG_OBJECT (demux,
+        "Waiting for audio stream pad to come up before we can push tags");
+    return;
+  }
+  if (demux->has_video && !demux->video_pad) {
+    GST_DEBUG_OBJECT (demux,
+        "Waiting for video stream pad to come up before we can push tags");
+    return;
+  }
+  if (demux->taglist) {
+    GST_DEBUG_OBJECT (demux, "pushing tags out %" GST_PTR_FORMAT,
+        demux->taglist);
+    gst_element_found_tags (GST_ELEMENT (demux), demux->taglist);
+    demux->taglist = gst_tag_list_new ();
+    demux->push_tags = FALSE;
+  }
+}
+
 static GstFlowReturn
 gst_flv_demux_parse_tag_audio (GstFlvDemux * demux, GstBuffer * buffer)
 {
@@ -795,6 +835,7 @@ gst_flv_demux_parse_tag_audio (GstFlvDemux * demux, GstBuffer * buffer)
       GST_DEBUG_OBJECT (demux, "emitting no more pads");
       gst_element_no_more_pads (GST_ELEMENT (demux));
       demux->no_more_pads = TRUE;
+      demux->push_tags = TRUE;
     }
   }
 
@@ -812,18 +853,8 @@ gst_flv_demux_parse_tag_audio (GstFlvDemux * demux, GstBuffer * buffer)
   }
 
   /* Push taglist if present */
-  if ((demux->has_audio && !demux->audio_pad) &&
-      (demux->has_video && !demux->video_pad)) {
-    GST_DEBUG_OBJECT (demux, "we are still waiting for a stream to come up "
-        "before we can push tags");
-  } else {
-    if (demux->taglist && demux->push_tags) {
-      GST_DEBUG_OBJECT (demux, "pushing tags out");
-      gst_element_found_tags (GST_ELEMENT (demux), demux->taglist);
-      demux->taglist = gst_tag_list_new ();
-      demux->push_tags = FALSE;
-    }
-  }
+  if (G_UNLIKELY (demux->push_tags))
+    gst_flv_demux_push_tags (demux);
 
   /* Check if we have anything to push */
   if (demux->tag_data_size <= codec_data) {
@@ -842,7 +873,7 @@ gst_flv_demux_parse_tag_audio (GstFlvDemux * demux, GstBuffer * buffer)
     switch (aac_packet_type) {
       case 0:
       {
-        /* AudioSpecificConfic data */
+        /* AudioSpecificConfig data */
         GST_LOG_OBJECT (demux, "got an AAC codec data packet");
         if (demux->audio_codec_data) {
           gst_buffer_unref (demux->audio_codec_data);
@@ -933,6 +964,7 @@ gst_flv_demux_parse_tag_audio (GstFlvDemux * demux, GstBuffer * buffer)
         " after 6 seconds of audio");
     gst_element_no_more_pads (GST_ELEMENT_CAST (demux));
     demux->no_more_pads = TRUE;
+    demux->push_tags = TRUE;
   }
 
   /* Push downstream */
@@ -968,29 +1000,24 @@ gst_flv_demux_video_negotiate (GstFlvDemux * demux, guint32 codec_tag)
 {
   gboolean ret = FALSE;
   GstCaps *caps = NULL;
-  const gchar *codec_name = NULL;
+  gchar *codec_name = NULL;
 
   /* Generate caps for that pad */
   switch (codec_tag) {
     case 2:
       caps = gst_caps_new_simple ("video/x-flash-video", NULL);
-      codec_name = "Sorenson Video";
       break;
     case 3:
       caps = gst_caps_new_simple ("video/x-flash-screen", NULL);
-      codec_name = "Flash Screen Video";
       break;
     case 4:
       caps = gst_caps_new_simple ("video/x-vp6-flash", NULL);
-      codec_name = "On2 VP6 Video";
       break;
     case 5:
       caps = gst_caps_new_simple ("video/x-vp6-alpha", NULL);
-      codec_name = "On2 VP6 Video with alpha channel";
       break;
     case 7:
       caps = gst_caps_new_simple ("video/x-h264", NULL);
-      codec_name = "H.264/AVC Video";
       break;
     default:
       GST_WARNING_OBJECT (demux, "unsupported video codec tag %u", codec_tag);
@@ -1015,11 +1042,14 @@ gst_flv_demux_video_negotiate (GstFlvDemux * demux, guint32 codec_tag)
     /* Store the caps we have set */
     demux->video_codec_tag = codec_tag;
 
+    codec_name = gst_pb_utils_get_codec_description (caps);
+
     if (codec_name) {
       if (demux->taglist == NULL)
         demux->taglist = gst_tag_list_new ();
       gst_tag_list_add (demux->taglist, GST_TAG_MERGE_REPLACE,
           GST_TAG_VIDEO_CODEC, codec_name, NULL);
+      g_free (codec_name);
     }
 
     GST_DEBUG_OBJECT (demux->video_pad, "successfully negotiated caps %"
@@ -1148,6 +1178,7 @@ gst_flv_demux_parse_tag_video (GstFlvDemux * demux, GstBuffer * buffer)
       GST_DEBUG_OBJECT (demux, "emitting no more pads");
       gst_element_no_more_pads (GST_ELEMENT (demux));
       demux->no_more_pads = TRUE;
+      demux->push_tags = TRUE;
     }
   }
 
@@ -1167,18 +1198,8 @@ gst_flv_demux_parse_tag_video (GstFlvDemux * demux, GstBuffer * buffer)
   }
 
   /* Push taglist if present */
-  if ((demux->has_audio && !demux->audio_pad) &&
-      (demux->has_video && !demux->video_pad)) {
-    GST_DEBUG_OBJECT (demux, "we are still waiting for a stream to come up "
-        "before we can push tags");
-  } else {
-    if (demux->taglist && demux->push_tags) {
-      GST_DEBUG_OBJECT (demux, "pushing tags out");
-      gst_element_found_tags (GST_ELEMENT (demux), demux->taglist);
-      demux->taglist = gst_tag_list_new ();
-      demux->push_tags = FALSE;
-    }
-  }
+  if (G_UNLIKELY (demux->push_tags))
+    gst_flv_demux_push_tags (demux);
 
   /* Check if we have anything to push */
   if (demux->tag_data_size <= codec_data) {
@@ -1291,6 +1312,7 @@ gst_flv_demux_parse_tag_video (GstFlvDemux * demux, GstBuffer * buffer)
         " after 6 seconds of video");
     gst_element_no_more_pads (GST_ELEMENT_CAST (demux));
     demux->no_more_pads = TRUE;
+    demux->push_tags = TRUE;
   }
 
   /* Push downstream */
