@@ -29,14 +29,6 @@
 GST_DEBUG_CATEGORY_STATIC (rtpgsmdepay_debug);
 #define GST_CAT_DEFAULT (rtpgsmdepay_debug)
 
-/* elementfactory information */
-static GstElementDetails gst_rtp_gsmdepay_details = {
-  "RTP GSM depayloader",
-  "Codec/Depayloader/Network",
-  "Extracts GSM audio from RTP packets",
-  "Zeeshan Ali <zeenix@gmail.com>"
-};
-
 /* RTPGSMDepay signals and args */
 enum
 {
@@ -82,7 +74,9 @@ gst_rtp_gsm_depay_base_init (gpointer klass)
       gst_static_pad_template_get (&gst_rtp_gsm_depay_src_template));
   gst_element_class_add_pad_template (element_class,
       gst_static_pad_template_get (&gst_rtp_gsm_depay_sink_template));
-  gst_element_class_set_details (element_class, &gst_rtp_gsmdepay_details);
+  gst_element_class_set_details_simple (element_class, "RTP GSM depayloader",
+      "Codec/Depayloader/Network/RTP",
+      "Extracts GSM audio from RTP packets", "Zeeshan Ali <zeenix@gmail.com>");
 }
 
 static void
@@ -154,5 +148,5 @@ gboolean
 gst_rtp_gsm_depay_plugin_init (GstPlugin * plugin)
 {
   return gst_element_register (plugin, "rtpgsmdepay",
-      GST_RANK_MARGINAL, GST_TYPE_RTP_GSM_DEPAY);
+      GST_RANK_SECONDARY, GST_TYPE_RTP_GSM_DEPAY);
 }
