@@ -65,7 +65,7 @@ gst_rtp_mp2t_pay_base_init (gpointer klass)
   gst_element_class_add_pad_template (element_class,
       gst_static_pad_template_get (&gst_rtp_mp2t_pay_src_template));
   gst_element_class_set_details_simple (element_class,
-      "RTP MPEG2 Transport Stream payloader", "Codec/Payloader/Network",
+      "RTP MPEG2 Transport Stream payloader", "Codec/Payloader/Network/RTP",
       "Payload-encodes MPEG2 TS into RTP packets (RFC 2250)",
       "Wim Taymans <wim.taymans@gmail.com>");
 }
@@ -202,5 +202,5 @@ gboolean
 gst_rtp_mp2t_pay_plugin_init (GstPlugin * plugin)
 {
   return gst_element_register (plugin, "rtpmp2tpay",
-      GST_RANK_NONE, GST_TYPE_RTP_MP2T_PAY);
+      GST_RANK_SECONDARY, GST_TYPE_RTP_MP2T_PAY);
 }
