@@ -91,7 +91,7 @@ gst_rtp_qcelp_depay_base_init (gpointer klass)
       gst_static_pad_template_get (&gst_rtp_qcelp_depay_sink_template));
 
   gst_element_class_set_details_simple (element_class, "RTP QCELP depayloader",
-      "Codec/Depayloader/Network",
+      "Codec/Depayloader/Network/RTP",
       "Extracts QCELP (PureVoice) audio from RTP packets (RFC 2658)",
       "Wim Taymans <wim.taymans@gmail.com>");
 }
@@ -157,7 +157,7 @@ gst_rtp_qcelp_depay_setcaps (GstBaseRTPDepayload * depayload, GstCaps * caps)
   return res;
 }
 
-static gint frame_size[16] = {
+static const gint frame_size[16] = {
   1, 4, 8, 17, 35, -8, 0, 0,
   0, 0, 0, 0, 0, 0, 1, 0
 };
@@ -431,5 +431,5 @@ gboolean
 gst_rtp_qcelp_depay_plugin_init (GstPlugin * plugin)
 {
   return gst_element_register (plugin, "rtpqcelpdepay",
-      GST_RANK_MARGINAL, GST_TYPE_RTP_QCELP_DEPAY);
+      GST_RANK_SECONDARY, GST_TYPE_RTP_QCELP_DEPAY);
 }
